var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
import { DIRECTION_ALL, DIRECTION_HORIZONTAL, DIRECTION_NONE, DIRECTION_VERTICAL, Manager, Pan } from "@egjs/hammerjs";
import { $ } from "../utils";
import { convertInputType, createHammer, toAxis, UNIQUEKEY } from "./InputType";
export function getDirectionByAngle(angle, thresholdAngle) {
    if (thresholdAngle < 0 || thresholdAngle > 90) {
        return DIRECTION_NONE;
    }
    var toAngle = Math.abs(angle);
    return toAngle > thresholdAngle && toAngle < 180 - thresholdAngle ?
        DIRECTION_VERTICAL : DIRECTION_HORIZONTAL;
}
export function getNextOffset(speeds, deceleration) {
    var normalSpeed = Math.sqrt(speeds[0] * speeds[0] + speeds[1] * speeds[1]);
    var duration = Math.abs(normalSpeed / -deceleration);
    return [
        speeds[0] / 2 * duration,
        speeds[1] / 2 * duration,
    ];
}
export function useDirection(checkType, direction, userDirection) {
    if (userDirection) {
        return !!((direction === DIRECTION_ALL) ||
            ((direction & checkType) && (userDirection & checkType)));
    }
    else {
        return !!(direction & checkType);
    }
}
var PanInput = (function () {
    function PanInput(el, options) {
        this.axes = [];
        this.hammer = null;
        this.element = null;
        this.panRecognizer = null;
        if (typeof Manager === "undefined") {
            throw new Error("The Hammerjs must be loaded before eg.Axes.PanInput.\nhttp://hammerjs.github.io/");
        }
        this.element = $(el);
        this.options = __assign({
            inputType: ["touch", "mouse", "pointer"],
            scale: [1, 1],
            thresholdAngle: 45,
            threshold: 0,
            hammerManagerOptions: {
                cssProps: {
                    userSelect: "none",
                    touchSelect: "none",
                    touchCallout: "none",
                    userDrag: "none",
                },
            },
        }, options);
        this.onHammerInput = this.onHammerInput.bind(this);
        this.onPanmove = this.onPanmove.bind(this);
        this.onPanend = this.onPanend.bind(this);
    }
    PanInput.prototype.mapAxes = function (axes) {
        var useHorizontal = !!axes[0];
        var useVertical = !!axes[1];
        if (useHorizontal && useVertical) {
            this._direction = DIRECTION_ALL;
        }
        else if (useHorizontal) {
            this._direction = DIRECTION_HORIZONTAL;
        }
        else if (useVertical) {
            this._direction = DIRECTION_VERTICAL;
        }
        else {
            this._direction = DIRECTION_NONE;
        }
        this.axes = axes;
    };
    PanInput.prototype.connect = function (observer) {
        var hammerOption = {
            direction: this._direction,
            threshold: this.options.threshold,
        };
        if (this.hammer) {
            this.removeRecognizer();
            this.dettachEvent();
        }
        else {
            var keyValue = this.element[UNIQUEKEY];
            if (!keyValue) {
                keyValue = String(Math.round(Math.random() * new Date().getTime()));
            }
            var inputClass = convertInputType(this.options.inputType);
            if (!inputClass) {
                throw new Error("Wrong inputType parameter!");
            }
            this.hammer = createHammer(this.element, __assign({
                inputClass: inputClass,
            }, this.options.hammerManagerOptions));
            this.element[UNIQUEKEY] = keyValue;
        }
        this.panRecognizer = new Pan(hammerOption);
        this.hammer.add(this.panRecognizer);
        this.attachEvent(observer);
        return this;
    };
    PanInput.prototype.disconnect = function () {
        this.removeRecognizer();
        if (this.hammer) {
            this.dettachEvent();
        }
        this._direction = DIRECTION_NONE;
        return this;
    };
    PanInput.prototype.destroy = function () {
        this.disconnect();
        if (this.hammer && this.hammer.recognizers.length === 0) {
            this.hammer.destroy();
        }
        delete this.element[UNIQUEKEY];
        this.element = null;
        this.hammer = null;
    };
    PanInput.prototype.enable = function () {
        this.hammer && (this.hammer.get("pan").options.enable = true);
        return this;
    };
    PanInput.prototype.disable = function () {
        this.hammer && (this.hammer.get("pan").options.enable = false);
        return this;
    };
    PanInput.prototype.isEnable = function () {
        return !!(this.hammer && this.hammer.get("pan").options.enable);
    };
    PanInput.prototype.removeRecognizer = function () {
        if (this.hammer && this.panRecognizer) {
            this.hammer.remove(this.panRecognizer);
            this.panRecognizer = null;
        }
    };
    PanInput.prototype.onHammerInput = function (event) {
        if (this.isEnable()) {
            if (event.isFirst) {
                this.observer.hold(this, event);
            }
            else if (event.isFinal) {
                this.onPanend(event);
            }
        }
    };
    PanInput.prototype.onPanmove = function (event) {
        var userDirection = getDirectionByAngle(event.angle, this.options.thresholdAngle);
        var prevInput = this.hammer.session.prevInput;
        if (prevInput) {
            event.offsetX = event.deltaX - prevInput.deltaX;
            event.offsetY = event.deltaY - prevInput.deltaY;
        }
        else {
            event.offsetX = 0;
            event.offsetY = 0;
        }
        var offset = this.getOffset([event.offsetX, event.offsetY], [
            useDirection(DIRECTION_HORIZONTAL, this._direction, userDirection),
            useDirection(DIRECTION_VERTICAL, this._direction, userDirection),
        ]);
        var prevent = offset.some(function (v) { return v !== 0; });
        if (prevent) {
            event.srcEvent.preventDefault();
            event.srcEvent.stopPropagation();
        }
        event.preventSystemEvent = prevent;
        prevent && this.observer.change(this, event, toAxis(this.axes, offset));
    };
    PanInput.prototype.onPanend = function (event) {
        var offset = this.getOffset([
            Math.abs(event.velocityX) * (event.deltaX < 0 ? -1 : 1),
            Math.abs(event.velocityY) * (event.deltaY < 0 ? -1 : 1),
        ], [
            useDirection(DIRECTION_HORIZONTAL, this._direction),
            useDirection(DIRECTION_VERTICAL, this._direction),
        ]);
        offset = getNextOffset(offset, this.observer.options.deceleration);
        this.observer.release(this, event, toAxis(this.axes, offset));
    };
    PanInput.prototype.attachEvent = function (observer) {
        this.observer = observer;
        this.hammer.on("hammer.input", this.onHammerInput)
            .on("panstart panmove", this.onPanmove);
    };
    PanInput.prototype.dettachEvent = function () {
        this.hammer.off("hammer.input", this.onHammerInput)
            .off("panstart panmove", this.onPanmove);
        this.observer = null;
    };
    PanInput.prototype.getOffset = function (properties, direction) {
        var offset = [0, 0];
        var scale = this.options.scale;
        if (direction[0]) {
            offset[0] = (properties[0] * scale[0]);
        }
        if (direction[1]) {
            offset[1] = (properties[1] * scale[1]);
        }
        return offset;
    };
    return PanInput;
}());
export { PanInput };
//# sourceMappingURL=PanInput.js.map